import java.util.*;

/**
 * Generic stream transformation program.
 */
public final class MAMCodec {

  /** No instantiation for this stand-alone program **/
  private MAMCodec() {}

  /**
   * Supported codecs
   */
  static public enum CodecNames {
    BWTEnc, // Burrows-Wheeler transform encoding
    BWTDec, // Burrows-Wheeler transform decoding
    M2FEnc, // Move-to-front transfrom encoding
    M2FDec, // Move-to-front transform decoding
    AriEnc, // Simple arithmetic encoding
    AriDec, // Simple artihmetic decoding
    MAMEnc, // Our text encoding
    MAMDec, // Our text decoding
    ZIPEnc, // GZIP encoding
    ZIPDec  // GZIP decoding
  };

  static public void main(String[] argv) {
    if (argv.length < 1) {
       System.err.println("No codec specified.");
       System.err.println("Available codes are: "+
                          Arrays.toString(CodecNames.values()));
       System.exit(-1);
    }

    Codec      codec;
    CodecNames cName;

    try { cName = CodecNames.valueOf(argv[0]); }
    catch (IllegalArgumentException e) {
      System.err.println("Unsupported codec: "+argv[0]);
      System.exit(-2); 
      return; // Never called
    }
    // Create codec instance
    switch (cName) {
      case BWTEnc: codec = new BWTEncode();
                   break;
      case BWTDec: codec = new BWTDecode();
                   break;
      case M2FEnc: codec = new M2FEncode();
                   break;
      case M2FDec: codec = new M2FDecode();
                   break;
      case MAMEnc: codec = new MAMEncode(new PPMModel());
                   break;
      case MAMDec: codec = new MAMDecode(new PPMModel());
                   break;
      case AriEnc: codec = new MAMEncode();
                   break;
      case AriDec: codec = new MAMDecode();
                   break;
      case ZIPEnc: codec = new ZIPEncode();
                   break;
      case ZIPDec: codec = new ZIPDecode();
                   break;
      default: throw new IllegalStateException("Codec defined but not supported!");
    }

    // Data processing occurs here
    try {
      codec.processTransformation(System.in,
                                  System.out);
    }
    catch (Exception e) {
      System.err.println("Codec failed: "+e.getLocalizedMessage());
      e.printStackTrace();
      System.exit(-3);
    }
  }

}
