import java.io.*;
import java.util.*;

/**
 * Aritmetic coding for binomial and byte sequencies.
 */
public class MAMEncode implements Codec {

  private ArithmeticModel model;

  public MAMEncode() {
    this(new AdaptiveModel());
  }

  public MAMEncode(ArithmeticModel mod) {
    model = mod;
  }

  public void setModel(ArithmeticModel mod) {
    model = mod;
  }

  /**
   * This method encodes symbols to the output stream, one at a time.
   */
  /* -- Implementation details --
    The high count is taken to be exclusive, and the resulting range is
    highCounts[i] - lowCounts[i] + 1 for every symbol.
    lowCounts is the cumulative count of symbols below current one.
    highCounts is the cumulative count of symbols below current one plus current one.
    total is the cumulative count of all symbols.
    One symbol should be 'eos' - it should appear only once and at the end of input.
  */
  public synchronized void processTransformation(InputStream  in,
                                                 OutputStream out) throws IOException {
    model.reset();

    BinaryOutputStream bOut = new BinaryOutputStream(out);
    int   low          = 0; // the low bound on the current interval for coding
    int   high         = Integer.MAX_VALUE; // the high bound on the current interval for coding
    long  range;//current long-value range - based on the probabilities
    int   firstQuarter = high / 4 + 1; // 1/4 of the largest possible value plus one
    int   half         = 2 * firstQuarter; // 1/2 of the largest possible value
    int   thirdQuarter = 3 * firstQuarter;// 3/4 of the largest possible value
    int   j;
    int   seen         = 0;
    int   bitsToFollow = 0;
    long  tc;

    do {
      j=in.read(); // the symbol as int
      range = ((long)high - (long)low) + 1;
      tc    = model.totalCount();
      if (tc==0) throw new IllegalStateException(model.getClass().getName()+".totalCount() == 0");
      high  = low + (int)((range * model.highInterval(j)) / tc) - 1;
      low   = low + (int)((range * model.lowInterval(j))  / tc);
      if (high < 0)
        throw new IllegalArgumentException("Invalid input for the given model");
      while (true) {
        if (high < half) {
          bitPlusFollow(false, bOut, bitsToFollow);
          bitsToFollow = 0;
          low  = 2*low;
          high = 2*high + 1;
        } else if (low >= half) {
          bitPlusFollow(true, bOut, bitsToFollow);
          bitsToFollow = 0;
          low  = 2*(low - half);
          high = 2*(high - half) + 1;
        } else if (low >= firstQuarter && high < thirdQuarter) {
          ++bitsToFollow;
          low  = low - firstQuarter;
          high = high - firstQuarter;
          low  = 2*low;
          high = 2*high + 1;
        } else {
          break;
        }
      }
      model.updateModel(j);
    } while (j != -1);
    ++bitsToFollow;
    bitPlusFollow(low>=firstQuarter, bOut, bitsToFollow);
    bOut.flushBitBuffer();
    bOut.flush();
  }

  private void bitPlusFollow(boolean            outBit,
                             BinaryOutputStream out,
                             int                bitsToFollow) throws IOException {
    out.writeBit(outBit);
    while(bitsToFollow > 0){
      out.writeBit(!outBit);
      --bitsToFollow;
    }
  }

}
