import java.io.*;
import java.util.*;

/**
 * Miscellaneous utilities for MAMCodec.
 */
public final class MAMUtil {

  /** No instantiation for the library class */
  private MAMUtil() {}


  static public int[] setDefaultDistribution() {
    int[] counts      = new int[257];
    for (int j=0; j<counts.length; ++j)counts[j] = 1;
    int total = 0;
    for (int i=0; i<counts.length; ++i) {
        total    += counts[i];
        counts[i] = total;
    }
    return counts;
  }

  static public byte[] setChars() {
    byte[] chars = new byte[256];

    for (int i=Byte.MIN_VALUE; i<=Byte.MAX_VALUE; i++) {
        chars[i-Byte.MIN_VALUE] = (byte)i;
    }
    return chars;
  }

  /**
   * Defaults for move-to-front transformation.
   */
  static public byte[] getDefaultByteOrder() {
     byte[] table = new byte[256];
     int    v     = 9;

     for (int i=0; i<table.length; i++) {
         table[i] = (byte)v++;
         if (v > Byte.MAX_VALUE)
            v = Byte.MIN_VALUE;
     }
     return table;
  }

  static public int setFrequencyDistribution(int[] dist, int[] freqs) {
    int total = 0;
    int last  = -1;

    for (int i=0; i<freqs.length; i++) {
        total += freqs[i];
    }
    double rate = ((10000)/(double)total);
    for (int i=0; i<dist.length; i++) {
        if (dist[i] > 0) {
           dist[i] = Math.max(1, (int)(freqs[i]*rate));
           if (last > 0)
              dist[i] += last;
           last = dist[i];
        }
    }
    return last;
  }

  /**
   * String coding function that can be used for test
   * purposes. Character encodings may lead to misinterpretations
   * of the results.
   */
  static public String callCodec(Codec  codec,
                                 String input) {
    ByteArrayInputStream  iS;
    ByteArrayOutputStream oS;

    iS = new ByteArrayInputStream(input.getBytes());
    oS = new ByteArrayOutputStream(1);
    try { codec.processTransformation(iS, oS); }
    catch (IOException e) {
      throw new IllegalStateException("IOException from byte array structures: "+e.toString());
    }
    return oS.toString();
  }

  static public void callCodec(final Codec        codec,
                               final InputStream  in,
                               final OutputStream out) {
    Thread th = new Thread() {
      Codec        CODEC = codec;
      InputStream  IN    = in;
      OutputStream OUT   = out;
      public void run() {
        try {
          codec.processTransformation(IN, OUT);
          OUT.flush();
          OUT.close();
        }
        catch (Exception e) {
          System.err.println("Fatal problem in "+
                             CODEC.getClass().getName()+".processTransformation(): "+
                             e.getLocalizedMessage());
          System.err.println("Make sure that the input format is OK.\n");
          e.printStackTrace();
          System.exit(-1);
        }
      }
    };
    th.start();
  }

  /**
   * Read as much as possible until the end of stream.
   * This method may block and wait for further input.
   */
  static public int read(InputStream in, byte[] buffer) throws IOException {
    int gain = 0;
    int avail;

    while ((avail=in.read(buffer, gain, buffer.length-gain)) > 0) {
      gain += avail;
    }
    return gain;
  }

  static public byte[] read(InputStream in) throws IOException {
    ByteArrayOutputStream out    = new ByteArrayOutputStream();
    byte[]                buffer = new byte[1024];
    int                   avail;

    while ((avail=in.read(buffer)) > 0) {
      out.write(buffer, 0, avail);
    }
    return out.toByteArray();
  }

  static public int readInt(InputStream in) throws IOException {
    byte[] bytes = new byte[4];
    if (read(in, bytes) < 4)
       throw new EOFException("Premature end of stream");
    return toInt(bytes);
  }

  /**
   * Converts a 32 bit int into a array of four bytes.
   */
  static public byte[] toBytes(int number) {
    byte[] b = new byte[4];

    b[0] = (byte) ( number        & 0x00FF);
    b[1] = (byte) ((number >>  8) & 0x000000FF);
    b[2] = (byte) ((number >> 16) & 0x000000FF);
    b[3] = (byte) ((number >> 24) & 0x000000FF);
    return b;
  }

  /**
   * Converts four bytes to a 32 bit int.
   */
  static public int toInt(byte[] b) {
    return toInt(b, 0);
  }

  /**
   * Converts four bytes to a 32 bit int.
   * The bytes are read starting from the given position.
   */
  static public int toInt(byte[] b, int offset) {
    int value = 0;
    for (int i = 0; i < 4; i++) {
        value += (b[i+offset] & 0x000000FF) << (8*i);
    }
    return value;
  }

  /**
   * Converts one byte to a sequence of eight bytes of zeros and ones
   * corresponding the bit sequence.
   */
  static public byte[] toBinaryBytes(byte bits) {
    byte[] array = new byte[8];
    byte   b;

    for (int i=0; i<array.length; i++) {
        array[i] += (bits >>> 7-i) & 0x1;
    }
    return array;
  }

//  /** Gets cumulative symbol count of the sybol with the previous index
//    (probabitily other than 0 or for the first symbol) in encoding. */
//  static public int getPreviousCount(int[] counts, int index){
//    int temp = index;
//    while (true) {
//      if (temp == 0)
//         return 0;
//      if (counts[temp - 1] != 0)
//         return counts[temp - 1];
//      --temp;
//    }
//  }
//
//  /** Update counts by one.*/
//  static public int[] updateCounts(int[] counts, int index){
//    while (index < counts.length){
//      ++counts[index];
//      ++index;
//    }
//    if (counts[255] >= Integer.MAX_VALUE - 2) counts = rescale(counts);
//    return counts;
//  }
//  static public int[] rescale(int[] counts) {
//    int[] freqs = new int[counts.length];
//    for (int i = 1; i < freqs.length; ++i)
//      freqs[i] = (counts[i] - counts[i-1] + 1) / 2;
//    counts[0] = (counts[0] + 1)/2;
//    for (int i = 1; i < counts.length; ++i)
//      counts[i] = counts[i-1] + freqs[i];
//    return counts;
//  }
  /** The function must be called before the encoding.
    Following should hold: freq.length == symbols.length.
    Symbols which no probability have count 0.
    The sum of freq should not be over Integer.MAX_VALUE - 1*/
  static public int[] setInitialCounts( int[] freq, int[] symbols){
    int[] counts      = new int[257];
    int[] chars      = new int[257];
    int total         = Integer.MAX_VALUE - 1;
    int counter   = 0;
    for (byte i=Byte.MIN_VALUE; i<Byte.MAX_VALUE; i++) {
        chars[counter] += i;
        ++counter;
    }
    chars[255] = Byte.MAX_VALUE;
    chars[256] = 128;
    int tempTotal = 0;
    for (int k=0; k<freq.length; ++k) tempTotal = tempTotal + freq[k];
    for (int j=0; j<symbols.length; ++j) {
      counts[Arrays.binarySearch(chars, symbols[j])] = (int)((long)total*freq[j]/tempTotal);
    }
    total = 0;
    for (int i=0; i<counts.length; ++i) {
        total    += counts[i];
        counts[i] = total;
    }
    return counts;
  }
}
